package xbc.formula.engine.exp.datatype;

import java.math.BigDecimal;
import java.util.List;

import xbc.formula.engine.exp.exception.XbcExpParseException;
import xbc.formula.engine.exp.vo.XbcExpIntervalVo;

/**
 * 表达式树
 * 
 * @author leichangchun
 */
public class XbcExpExpTree {
	public int ek;   // ExpKind
	public int ok;   // OpKind
	public XbcExpExpTree lchild;
	public XbcExpExpTree mchild;
	public XbcExpExpTree rchild;
	public Object value;
	public List<XbcExpExpTree> params;

	public List<XbcExpIntervalVo> itvs; // 层差计算用

	public XbcExpExpTree body;

	public XbcExpExpTree next; // 下一个表达式
	
	public String ruleName;
	public List<XbcExpExpTree> ruleParams;

	public int linenum;

	public XbcExpExpTree(int linenum) {
		this.linenum = linenum;
	}
	
	public XbcExpExpTree(int linenum, int ok) {
		this.linenum = linenum;
		this.ek = XbcExpExpKind.OP_EK;
		this.ok = ok;
	}
	
	public static XbcExpExpTree newConstIntExptree(int linenum, String tokenContent, String expStr) {
		XbcExpExpTree exp = new XbcExpExpTree(linenum);
		exp.ek = XbcExpExpKind.CST_EK;
		exp.ok = XbcExpExpVarKind.INT_VK;
		try {
			int iv = Integer.valueOf(tokenContent).intValue();
			exp.value = iv;
		} catch(NumberFormatException nx) {
			throw new XbcExpParseException("无法将【" + tokenContent + "】转换成整数。" + " 公式: " + expStr);
		}
		return exp;
	}

	public static XbcExpExpTree newConstDblExptree(int linenum, String tokenContent, String expStr) {
		XbcExpExpTree exp = new XbcExpExpTree(linenum);
		exp.ek = XbcExpExpKind.CST_EK;
		exp.ok = XbcExpExpVarKind.DBL_VK;
		try {
			BigDecimal bv = new BigDecimal(tokenContent);
			exp.value = bv;
		} catch(NumberFormatException nx) {
			throw new XbcExpParseException("无法将【" + tokenContent + "】转换成浮点数。" + " 公式: " + expStr);
		}
		return exp;
	}

	public static XbcExpExpTree newConstBoolExptree(int linenum, boolean bv) {
		XbcExpExpTree exp = new XbcExpExpTree(linenum);
		exp.ek = XbcExpExpKind.CST_EK;
		exp.ok = XbcExpExpVarKind.BOOL_VK;
		exp.value = bv;
		return exp;
	}

	public static XbcExpExpTree newConstNullExptree(int linenum) {
		XbcExpExpTree exp = new XbcExpExpTree(linenum);
		exp.ek = XbcExpExpKind.CST_EK;
		exp.ok = XbcExpExpVarKind.UNK_VK;
		exp.value = null;
		return exp;
	}

	public static XbcExpExpTree newConstStrExptree(int linenum, String tokenContent) {
		XbcExpExpTree exp = new XbcExpExpTree(linenum);
		exp.ek = XbcExpExpKind.CST_EK;
		exp.ok = XbcExpExpVarKind.STR_VK;
		exp.value = tokenContent;
		return exp;
	}
	
	public static XbcExpExpTree newIdExptree(int linenum, String tokenContent) {
		XbcExpExpTree exp = new XbcExpExpTree(linenum);
		exp.ek = XbcExpExpKind.ID_EK;
		exp.value = tokenContent;
		return exp;
	}
}
