package xbc.formula.engine.exp.datatype;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

/**
 * Token定义
 * 
 * @author leichangchun
 */
public class XbcExpToken {
	@XbcExpTokenAnnotation(id="var", name="变量", value="var")
	public static final int EXPNAME_TT = 1;
	@XbcExpTokenAnnotation(id="int", name="整数", value="int")
	public static final int INT_TT = 2;
	@XbcExpTokenAnnotation(id="double", name="浮点数", value="double")
	public static final int DBL_TT = 3;
	@XbcExpTokenAnnotation(id="string", name="字符串", value="string")
	public static final int STR_TT = 4;
	@XbcExpTokenAnnotation(id="asn", name="等号", value="=")
	public static final int ASN_TT = 5;
	@XbcExpTokenAnnotation(id="mptasn", name="乘等号", value="*=")
	public static final int MPTASN_TT = 6;
	@XbcExpTokenAnnotation(id="divasn", name="除等号", value="/=")
	public static final int DIVASN_TT = 7;
	@XbcExpTokenAnnotation(id="modasn", name="取模等号", value="%=")
	public static final int MODASN_TT = 8;
	@XbcExpTokenAnnotation(id="plsasn", name="加等号", value="+=")
	public static final int PLSASN_TT = 9;
	@XbcExpTokenAnnotation(id="mnsasn", name="减等号", value="-=")
	public static final int MNSASN_TT = 10;
	@XbcExpTokenAnnotation(id="s2lasn", name="左移位等号", value="<<=")
	public static final int S2LASN_TT = 11;
	@XbcExpTokenAnnotation(id="s2rasn", name="右移位等号", value=">>=")
	public static final int S2RASN_TT = 12;
	@XbcExpTokenAnnotation(id="bandasn", name="与等号", value="&=")
	public static final int BANDASN_TT = 13;
	@XbcExpTokenAnnotation(id="bxorasn", name="异或等号", value="^=")
	public static final int BXORASN_TT = 14;
	@XbcExpTokenAnnotation(id="borasn", name="或等号", value="|=")
	public static final int BORASN_TT = 15;
	@XbcExpTokenAnnotation(id="mpt", name="乘号", value="*")
	public static final int MPT_TT = 16;
	@XbcExpTokenAnnotation(id="div", name="除号", value="/")
	public static final int DIV_TT = 17;
	@XbcExpTokenAnnotation(id="mod", name="取模", value="%")
	public static final int MOD_TT = 18;
	@XbcExpTokenAnnotation(id="pls", name="加号", value="+")
	public static final int PLS_TT = 19;
	@XbcExpTokenAnnotation(id="mns", name="减号", value="-")
	public static final int MNS_TT = 20;
	@XbcExpTokenAnnotation(id="s2l", name="左移位", value="<<")
	public static final int S2L_TT = 21;
	@XbcExpTokenAnnotation(id="s2r", name="右移位", value=">>")
	public static final int S2R_TT = 22;
	@XbcExpTokenAnnotation(id="band", name="与号", value="&")
	public static final int BAND_TT = 23;
	@XbcExpTokenAnnotation(id="bor", name="或", value="|")
	public static final int BOR_TT = 24;
	@XbcExpTokenAnnotation(id="bxor", name="异或", value="^")
	public static final int BXOR_TT = 25;
	@XbcExpTokenAnnotation(id="bnot", name="非", value="~")
	public static final int BNOT_TT = 26;
	@XbcExpTokenAnnotation(id="selfmns", name="自减", value="--")
	public static final int SELFMNS_TT = 27;
	@XbcExpTokenAnnotation(id="selfpls", name="自增", value="++")
	public static final int SELFPLS_TT = 28;
	@XbcExpTokenAnnotation(id="qry", name="问号", value="?")
	public static final int QRY_TT = 29;
	@XbcExpTokenAnnotation(id="cln", name="冒号", value=":")
	public static final int CLN_TT = 30;
	@XbcExpTokenAnnotation(id="cmm", name="逗号", value=",")
	public static final int CMM_TT = 31;
	@XbcExpTokenAnnotation(id="lprn", name="左括号", value="(")
	public static final int LPRN_TT = 32;
	@XbcExpTokenAnnotation(id="rprn", name="右括号", value=")")
	public static final int RPRN_TT = 33;
	@XbcExpTokenAnnotation(id="or", name="条件或", value="||")
	public static final int OR_TT = 34;
	@XbcExpTokenAnnotation(id="and", name="条件与", value="&&")
	public static final int AND_TT = 35;
	@XbcExpTokenAnnotation(id="eq", name="条件等", value="==")
	public static final int EQ_TT = 36;
	@XbcExpTokenAnnotation(id="neq", name="条件不等", value="!=")
	public static final int NEQ_TT = 37;
	@XbcExpTokenAnnotation(id="lt", name="小于", value="<")
	public static final int LT_TT = 38;
	@XbcExpTokenAnnotation(id="gt", name="大于", value=">")
	public static final int GT_TT = 39;
	@XbcExpTokenAnnotation(id="leq", name="小于等于", value="<=")
	public static final int LEQ_TT = 40;
	@XbcExpTokenAnnotation(id="geq", name="大于等于", value=">=")
	public static final int GEQ_TT = 41;
	@XbcExpTokenAnnotation(id="not", name="条件非", value="!")
	public static final int NOT_TT = 42;
	@XbcExpTokenAnnotation(id="lsqb", name="左大括号", value="{")
	public static final int LSQB_TT = 43;
	@XbcExpTokenAnnotation(id="rsqb", name="右大括号", value="}")
	public static final int RSQB_TT = 44;
	@XbcExpTokenAnnotation(id="lbkt", name="左中括号", value="[")
	public static final int LBKT_TT = 45;
	@XbcExpTokenAnnotation(id="rbkt", name="右中括号", value="]")
	public static final int RBKT_TT = 46;
	@XbcExpTokenAnnotation(id="semi", name="分号", value=";")
	public static final int SEMI_TT = 47;
	@XbcExpTokenAnnotation(id="div0", name="div0", value="div0")
	public static final int DIV0_TT = 48;
	@XbcExpTokenAnnotation(id="period", name="点号", value=".")
	public static final int PRD_TT = 49;
	@XbcExpTokenAnnotation(id="at", name="at", value="@")
	public static final int AT_TT = 50;
	@XbcExpTokenAnnotation(id="if", name="if", value="if")
	public static final int IF_TT = 60;
	@XbcExpTokenAnnotation(id="for", name="for", value="for")
	public static final int FOR_TT = 61;
	@XbcExpTokenAnnotation(id="while", name="while", value="while")
	public static final int WHILE_TT = 62;
	@XbcExpTokenAnnotation(id="continue", name="continue", value="continue")
	public static final int CONTINUE_TT = 63;
	@XbcExpTokenAnnotation(id="break", name="break", value="break")
	public static final int BREAK_TT = 64;
	@XbcExpTokenAnnotation(id="else", name="else", value="else")
	public static final int ELSE_TT = 65;
	@XbcExpTokenAnnotation(id="exit", name="exit", value="exit")
	public static final int EXIT_TT = 66;
	@XbcExpTokenAnnotation(id="new", name="new", value="new")
	public static final int NEW_TT = 67;
	@XbcExpTokenAnnotation(id="invoke", name="invoke", value="invoke")
	public static final int INVOKE_TT = 68;
	@XbcExpTokenAnnotation(id="eof", name="结束符", value="eof")
	public static final int EOF_TT = 99;
	
	private static Map<Integer, XbcExpTokenAnnotation> tokenMap = null;
	
	public static String getTokenName(int token) {
		if(tokenMap == null) {
			initTokenMap();
		}
		XbcExpTokenAnnotation a = tokenMap.get(token);
		if(a != null) {
			return a.name();
		}
		return "未知";
	}
	
	public static int lookupReservedWord(String rw) {
		if(rw.equals("if")) {
			return IF_TT;
		}
		if(rw.equals("for")) {
			return FOR_TT;
		}
		if(rw.equals("while")) {
			return WHILE_TT;
		}
		if(rw.equals("continue")) {
			return CONTINUE_TT;
		}
		if(rw.equals("break")) {
			return BREAK_TT;
		}
		return 0;
	}
	
	private synchronized static void initTokenMap() {
		if(tokenMap != null) {
			return;
		}
		tokenMap = new HashMap<>();
		Field[] fields = XbcExpToken.class.getDeclaredFields();
		for(Field field : fields) {
			XbcExpTokenAnnotation a = field.getAnnotation(XbcExpTokenAnnotation.class);
			if(a == null) {
				continue;
			}
			try {
				int t = field.getInt(null);
				tokenMap.put(t, a);
			} catch (IllegalArgumentException | IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	}

}
