package xbc.formula.engine.exp.exec;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import xbc.formula.engine.exp.datatype.XbcExpExpKind;
import xbc.formula.engine.exp.datatype.XbcExpExpOpKind;
import xbc.formula.engine.exp.datatype.XbcExpExpTree;
import xbc.formula.engine.exp.exception.XbcExpExecException;
import xbc.formula.engine.exp.util.XbcExpNumericUtil;
import xbc.formula.engine.exp.vo.XbcExpRuleVo;
import xbc.formula.engine.exp.vo.XbcExpResultVo;
import xbc.formula.engine.exp.vo.XbcExpRuleParamVo;

/**
 * 表达式执行类
 * 
 * @author leichangchun
 */
public class XbcExpCalculateExp {

	private String expStr;
	
	private int weishu;
	
	private RoundingMode roundMode;
	
	private Map<String, XbcExpResultVo> indicatorMap;
	
	private XbcExpCalculatorExp2Op calculatorExp2Op;
	
	private XbcExpCalculatorExpItv calculatorExpItv;
	
	private XbcExpCalculatorExpFunc calculatorExpFunc;
	
	private XbcExpCalculatorExpNew calculatorExpNew;
	
	private Map<String, XbcExpResultVo> varMap;
	
	private List<BigDecimal> div0List;
	
	private List<XbcExpRuleVo> ruleVos;
	
	public XbcExpCalculateExp(int weishu, RoundingMode roundMode, String expStr, Map<String, XbcExpResultVo> indicatorMap, List<XbcExpRuleVo> ruleVos, Object[] formulaParams) {
		if(indicatorMap == null) {
			indicatorMap = new HashMap<>();
		}
		this.weishu = weishu;
		this.roundMode = roundMode;
		varMap = new HashMap<>();
		div0List = new ArrayList<>();
		this.ruleVos = ruleVos;
		this.expStr = expStr;
		this.indicatorMap = indicatorMap;
		calculatorExp2Op = new XbcExpCalculatorExp2Op(this, weishu, roundMode, expStr);
		calculatorExpItv = new XbcExpCalculatorExpItv(this, weishu, roundMode, expStr);
		calculatorExpFunc = new XbcExpCalculatorExpFunc(this, weishu, roundMode, expStr, formulaParams);
		calculatorExpNew = new XbcExpCalculatorExpNew(calculatorExpFunc, expStr);
	}

	public Object calculate(XbcExpExpTree exp) {
		Object ret = null;
		try {
			ret = calculateEx(exp);
		} catch(XbcExpExitException x) {
			ret = x.getResult().value;
		}
		return ret;
	}

	public Object calculateEx(XbcExpExpTree exp) {
		if (exp == null) {
			return null;
		}
		XbcExpResultVo result = new XbcExpResultVo();
		while (exp != null) { // 执行完所有表达式，输出最后一个表达式的值
			if (exp.ek != XbcExpExpKind.OP_EK || exp.ok != XbcExpExpOpKind.NULL_OK) {
				try {
					calcExp(exp, result);
				} catch (XbcExpExitException x) {
					// result.value = x.getResult().value;
					throw x;
				}
			}
			exp = exp.next;
		}
		return result.value;
	}

	protected void calcExp(XbcExpExpTree exp, XbcExpResultVo result) {
		if(exp.ek == XbcExpExpKind.OP_EK) {
			calcOpExp(exp, result);
			return;
		}
		if(exp.ek == XbcExpExpKind.ID_EK) {
			String key = (String)exp.value;
			XbcExpResultVo pv = indicatorMap.get(key);
			if(pv == null) {
				if(varMap.containsKey(key)) {
					XbcExpResultVo varVal = varMap.get(key);
					result.value = varVal.value;
					return;
				}
				throw new XbcExpExecException(exp.linenum + "行: 需要传入一个名为【" + key + "】的参数。公式: " + expStr);
			}
			result.value = pv.value;
			return;
		}
		result.value = exp.value;
		return;
	}

	private static int isAsnExOk(int ok) {
		switch (ok) {
		case XbcExpExpOpKind.MPTASN_OK:
			return XbcExpExpOpKind.MPT_OK;
		case XbcExpExpOpKind.DIVASN_OK:
			return XbcExpExpOpKind.DIV_OK;
		case XbcExpExpOpKind.MODASN_OK:
			return XbcExpExpOpKind.MOD_OK;
		case XbcExpExpOpKind.PLSASN_OK:
			return XbcExpExpOpKind.PLS_OK;
		case XbcExpExpOpKind.MNSASN_OK:
			return XbcExpExpOpKind.MNS_OK;
		case XbcExpExpOpKind.S2LASN_OK:
			return XbcExpExpOpKind.S2L_OK;
		case XbcExpExpOpKind.S2RASN_OK:
			return XbcExpExpOpKind.S2R_OK;
		case XbcExpExpOpKind.BANDASN_OK:
			return XbcExpExpOpKind.BAND_OK;
		case XbcExpExpOpKind.BXORASN_OK:
			return XbcExpExpOpKind.BXOR_OK;
		case XbcExpExpOpKind.BORASN_OK:
			return XbcExpExpOpKind.BOR_OK;
		}
		return 0;
	}

	private boolean calcAsnExExp1(XbcExpExpTree exp, XbcExpResultVo result) {
		int asnEx = isAsnExOk(exp.ok);
		if(asnEx > 0) {
			calcAsnExExp(exp, result, asnEx);
		}
		return false;
	}
	
	private void calcOpExp(XbcExpExpTree exp, XbcExpResultVo result) {
		if(calcAsnExExp1(exp, result)) {
			return;
		}
		switch(exp.ok) {
		case XbcExpExpOpKind.ASN_OK:
			calcAsnExp(exp, result);
			break;
		case XbcExpExpOpKind.QRY_OK:
			calcQryExp(exp, result);
			break;
	    case XbcExpExpOpKind.MPT_OK:
	    case XbcExpExpOpKind.DIV_OK:
	    case XbcExpExpOpKind.MOD_OK:
	    case XbcExpExpOpKind.PLS_OK:
	    case XbcExpExpOpKind.MNS_OK:
	    case XbcExpExpOpKind.S2L_OK:
	    case XbcExpExpOpKind.S2R_OK:
	    case XbcExpExpOpKind.BAND_OK:
	    case XbcExpExpOpKind.BXOR_OK:
	    case XbcExpExpOpKind.BOR_OK:
	    case XbcExpExpOpKind.EQ_OK:
	    case XbcExpExpOpKind.NEQ_OK:
	    case XbcExpExpOpKind.LT_OK:
	    case XbcExpExpOpKind.GT_OK:
	    case XbcExpExpOpKind.LEQ_OK:
	    case XbcExpExpOpKind.GEQ_OK:
	    	calculatorExp2Op.calc2OperandsExp(exp, result);
	    	break;
	    
	    case XbcExpExpOpKind.OR_OK:
	    	calcOrExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.AND_OK:
	    	calcAndExp(exp, result);
	    	break;
	    
	    case XbcExpExpOpKind.BNOT_OK:
	    	calcBnotExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.NOT_OK:
	    	calcNotExp(exp, result);
	    	break;

	    case XbcExpExpOpKind.PREINC_OK:
	    	calcPreincExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.AFTINC_OK:
	    	calcAftincExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.PREDEC_OK:
	    	calcPredecExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.AFTDEC_OK:
	    	calcAftdecExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.FUNC_OK:
	    	calculatorExpFunc.calcFuncExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.ITV_OK:
	    	calculatorExpItv.calcItvExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.DIV0_OK:
	    	calcDiv0Exp(exp, result);
	    	break;
	    case XbcExpExpOpKind.EXIT_OK:
	    	calcExitExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.NEW_OK:
	    	calculatorExpNew.calcNewExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.INVOKE_OK:
	    	calculatorExpNew.calcInvokeExp(exp, result);
	    	break;

	    case XbcExpExpOpKind.PRD_OK:
	    	calcPrdExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.FOR_OK:
	    	calcForExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.BREAK_OK:
	    	calcBreakExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.CONTINUE_OK:
	    	calcContinueExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.WHILE_OK:
	    	calcWhileExp(exp, result);
	    	break;
	    case XbcExpExpOpKind.IF_OK:
	    	calcIfExp(exp, result);
	    	break;

	    case XbcExpExpOpKind.AT_OK:
	    	calcAtExp(exp, result);
	    	break;
		}
	}
	
	private void setValue2Obj(Object ov, String fieldName, Object v) {
		Class<?> c = ov.getClass();
		try {
			Field field = c.getDeclaredField(fieldName);
			field.setAccessible(true);
			field.set(ov, v);
		} catch (NoSuchFieldException | SecurityException | IllegalArgumentException | IllegalAccessException e) {
			//e.printStackTrace();
		}
	}

	private void calcAtExp(XbcExpExpTree exp, XbcExpResultVo result) {
		List<XbcExpRuleParamVo> rpvs = null;
		if(ruleVos != null && exp.ruleParams != null && exp.ruleParams.size() > 0) {
			rpvs = new ArrayList<>();
			List<XbcExpExpTree> rps = exp.ruleParams;
			XbcExpResultVo r = new XbcExpResultVo();
			for(int k = 0;k < rps.size();k ++) {
				XbcExpExpTree rp = rps.get(k);
				calcExp(rp, r);
				XbcExpRuleParamVo rpv = new XbcExpRuleParamVo();
				rpv.setPname((String)(rp.value));
				rpv.setPindex(k + 1);
				rpv.setValue(r.value);
				rpvs.add(rpv);
			}
		}
		try {
			calcExp(exp.lchild, result);
		} finally {
			if(ruleVos == null) {
				return;
			}
			List<XbcExpExpTree> params = exp.params;
			if(params == null || params.size() == 0) {
				return;
			}
			XbcExpRuleVo rv = new XbcExpRuleVo();
			XbcExpResultVo r = new XbcExpResultVo();
			boolean record = true;
			for(XbcExpExpTree pe : params) {
				calcExp(pe, r);
				String fn = pe.ruleName;
				if(fn.equals("record")) {
					if(result.value == null) {
						if(r.value == null) {
							record = true;
						} else {
							record = false;
						}
					} else {
						if(r.value == null) {
							record = false;
						} else if(result.value.equals(r.value)) {
							record = true;
						} else {
							record = false;
						}
					}
				} else if(fn.equals("notrecord")) {
					if(result.value == null) {
						if(r.value == null) {
							record = false;
						} else {
							record = true;
						}
					} else {
						if(r.value == null) {
							record = true;
						} else if(result.value.equals(r.value)) {
							record = false;
						} else {
							record = true;
						}
					}
				} else {
					setValue2Obj(rv, fn, r.value);
				}
			}
			rv.setValue(result.value);
			rv.setRpvs(rpvs);
			if(record) {
				ruleVos.add(rv);
			}
		}
	}
	
	private void calcExitExp(XbcExpExpTree exp, XbcExpResultVo result) {
		calcExp(exp.lchild, result);
		throw new XbcExpExitException(result);
	}
	
	private void calcIfExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo pr = new XbcExpResultVo();
		calcExp(exp.lchild, pr);
		boolean ifResult = false;
		if(pr.value != null && (pr.value.getClass() == Boolean.class || pr.value.getClass() == boolean.class)) {
			if(pr.value.getClass() == Boolean.class) {
				ifResult = ((Boolean)(pr.value)).booleanValue();
			} else {
				ifResult = (boolean)(pr.value);
			}
		} else {
			throw new XbcExpExecException(exp.linenum + "行: if语句的条件表达式的结果不是bool类型。公式: " + expStr);
		}
		Object ov = null;
		if(ifResult) {
			ov = calculateEx(exp.mchild);
		} else if(exp.rchild != null) {
			ov = calculateEx(exp.rchild);
		}
		result.value = ov;
	}
	
	private void calcBreakExp(XbcExpExpTree exp, XbcExpResultVo result) {
		throw new XbcExpBreakException();
	}

	private void calcContinueExp(XbcExpExpTree exp, XbcExpResultVo result) {
		throw new XbcExpContinueException();
	}
	
	private void calcWhileExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo pr = new XbcExpResultVo();
		while(true) {
			calcExp(exp.lchild, pr);
			if(pr.value != null && (pr.value.getClass() == Boolean.class || pr.value.getClass() == boolean.class)) {
				if(pr.value.getClass() == Boolean.class) {
					Boolean bv = (Boolean)(pr.value);
					if(!bv) {
						break;
					}
				} else {
					boolean bv = (boolean)(pr.value);
					if(!bv) {
						break;
					}
				}
			} else {
				throw new XbcExpExecException(exp.linenum + "行: while语句的条件表达式的结果不是bool类型。公式: " + expStr);
			}
			try {
				if(exp.body != null) {
					result.value = calculateEx(exp.body);
				}
			} catch(XbcExpBreakException cx) {
				break;
			} catch(XbcExpContinueException cx) {
				// continue;
			}
		}
	}
	
	private void calcForExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo pr = new XbcExpResultVo();
		if(exp.lchild != null) {
			calcExp(exp.lchild, pr);
		}
		while(true) {
			if(exp.mchild != null) {
				calcExp(exp.mchild, pr);
				if(pr.value != null && (pr.value.getClass() == Boolean.class || pr.value.getClass() == boolean.class)) {
					if(pr.value.getClass() == Boolean.class) {
						Boolean bv = (Boolean)(pr.value);
						if(!bv) {
							break;
						}
					} else {
						boolean bv = (boolean)(pr.value);
						if(!bv) {
							break;
						}
					}
				} else {
					throw new XbcExpExecException(exp.linenum + "行: for语句的中间表达式的结果不是bool类型。公式: " + expStr);
				}
			}
			try {
				if(exp.body != null) {
					result.value = calculateEx(exp.body);
				}
			} catch(XbcExpBreakException cx) {
				break;
			} catch(XbcExpContinueException cx) {
				// continue;
			}
			if(exp.rchild != null) {
				calcExp(exp.rchild, pr);
			}
		}
	}
	
	private void calcPrdExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpExpTree lchild = exp.lchild;
		String fieldName = (String)exp.value;
		XbcExpResultVo pr = new XbcExpResultVo();
		calcExp(lchild, pr);
		if(pr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: " + fieldName + " 的前面为空值。公式: " + expStr);
		}
		try {
			Field field = pr.value.getClass().getDeclaredField(fieldName);
			if(field == null) {
				throw new XbcExpExecException(exp.linenum + "行: " + fieldName + " 不是成员变量。");
			}
			field.setAccessible(true);
			Object ret = field.get(pr.value);
			result.value = ret;
		} catch (NoSuchFieldException nx) {
			throw new XbcExpExecException(exp.linenum + "行: " + fieldName + " 不是成员变量。");
		} catch(SecurityException | IllegalArgumentException | IllegalAccessException sx) {
			throw new XbcExpExecException(sx, exp.linenum + "行: " + fieldName + "执行出错。");
		}
	}

	private void calcDiv0Exp(XbcExpExpTree exp, XbcExpResultVo result) {
		calcExp(exp.lchild, result);
		BigDecimal bv = XbcExpNumericUtil.translate2BigDecimal(result.value, weishu, roundMode, expStr);
		if(BigDecimal.ZERO.compareTo(bv) == 0) {
			if(div0List.size() > 0) {
				div0List.remove(div0List.size() - 1);
			}
		} else {
			div0List.add(bv);
		}
	}
	
	protected BigDecimal getDiv0Val() {
		if(div0List.size() == 0) {
			return null;
		}
		return div0List.get(div0List.size() - 1);
	}

	private void calcAsnExExp(XbcExpExpTree exp, XbcExpResultVo result, int asnEx) {
		XbcExpExpTree expEx = new XbcExpExpTree(exp.linenum, XbcExpExpOpKind.ASN_OK);
		expEx.lchild = exp.lchild;
		expEx.rchild = new XbcExpExpTree(exp.linenum, asnEx);
		expEx.rchild.lchild = exp.lchild;
		expEx.rchild.rchild = exp.rchild;
		calcAsnExp(expEx, result);
	}
	
	private XbcExpResultVo getUnaryVn(XbcExpExpTree exp) {
		XbcExpResultVo result = null;
		if(exp.ek == XbcExpExpKind.ID_EK) {
			String key = (String)exp.value;
			result = indicatorMap.get(key);
			if(result == null) {
				if(varMap.containsKey(key)) {
					result = varMap.get(key);
				} else {
					result = new XbcExpResultVo();
				}
				return result;
			}
			return result;
		}
		if(exp.ek == XbcExpExpKind.CST_EK) {
			throw new XbcExpExecException(exp.linenum + "行: 不能为常量赋值。公式: " + expStr);
		} else {
			throw new XbcExpExecException(exp.linenum + "行: 暂不支持为表达式赋值。公式: " + expStr);
		}
	}

	private void calcAsnExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = getUnaryVn(exp.lchild);
		XbcExpResultVo rr = new XbcExpResultVo();
		calcExp(exp.rchild, rr);
		lr.value = rr.value;
		result.value = rr.value;
		String key = (String)(exp.lchild.value);
		if(varMap.containsKey(key)) {
			varMap.remove(key);
		}
		varMap.put(key, rr);
	}

	private void calcQryExp(XbcExpExpTree exp, XbcExpResultVo result) {
		boolean bv = calculatorExp2Op.testExpVal(exp.lchild);
		if(bv) {
			calcExp(exp.rchild.lchild, result);
		} else {
			calcExp(exp.rchild.rchild, result);
		}
	}

	private void calcOrExp(XbcExpExpTree exp, XbcExpResultVo result) {
		boolean bv = calculatorExp2Op.testExpVal(exp.lchild);
		if(bv) {
			result.value = true;
			return;
		}
		bv = calculatorExp2Op.testExpVal(exp.rchild);
		result.value = bv;
	}

	private void calcAndExp(XbcExpExpTree exp, XbcExpResultVo result) {
		boolean bv = calculatorExp2Op.testExpVal(exp.lchild);
		if(!bv) {
			result.value = false;
			return;
		}
		bv = calculatorExp2Op.testExpVal(exp.rchild);
		result.value = bv;
	}

	private void calcBnotExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = new XbcExpResultVo();
		calcExp(exp.lchild, lr);
		if(lr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对空值(null)做按位或操作。公式: " + expStr);
		}
		if(lr.value.getClass() != Integer.class && lr.value.getClass() != int.class) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对非整数做按位或操作。公式: " + expStr);
		}
		int iv = (int)lr.value;
		result.value = ~iv;
	}

	private void calcNotExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = new XbcExpResultVo();
		calcExp(exp.lchild, lr);
		if(lr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对空值(null)求非。公式: " + expStr);
		}
		if(lr.value.getClass() != Boolean.class && lr.value.getClass() != boolean.class) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对非布尔值求非。公式: " + expStr);
		}
		boolean bv = (boolean)lr.value;
		result.value = !bv;
	}

	private void calcPreincExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = getUnaryVn(exp.lchild);
		if(lr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对空值(null)做自增操作。公式: " + expStr);
		}
		if(lr.value.getClass() != Integer.class && lr.value.getClass() != int.class) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对非整数做自增操作。公式: " + expStr);
		}
		int iv = (int)lr.value;
		result.value = iv + 1;
		lr.value = iv + 1;
	}

	private void calcAftincExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = getUnaryVn(exp.lchild);
		if(lr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对空值(null)做自增操作。公式: " + expStr);
		}
		if(lr.value.getClass() != Integer.class && lr.value.getClass() != int.class) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对非整数做自增操作。公式: " + expStr);
		}
		int iv = (int)lr.value;
		result.value = iv;
		lr.value = iv + 1;
	}

	private void calcPredecExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = getUnaryVn(exp.lchild);
		if(lr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对空值(null)做自减操作。公式: " + expStr);
		}
		if(lr.value.getClass() != Integer.class && lr.value.getClass() != int.class) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对非整数做自减操作。公式: " + expStr);
		}
		int iv = (int)lr.value;
		result.value = iv - 1;
		lr.value = iv - 1;
	}

	private void calcAftdecExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = getUnaryVn(exp.lchild);
		if(lr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对空值(null)做自减操作。公式: " + expStr);
		}
		if(lr.value.getClass() != Integer.class && lr.value.getClass() != int.class) {
			throw new XbcExpExecException(exp.linenum + "行: 不能对非整数做自减操作。公式: " + expStr);
		}
		int iv = (int)lr.value;
		result.value = iv;
		lr.value = iv - 1;
	}
	
}
