package xbc.formula.engine.exp.exec;

import java.math.BigDecimal;
import java.math.RoundingMode;

import xbc.formula.engine.exp.datatype.XbcExpExpOpKind;
import xbc.formula.engine.exp.datatype.XbcExpExpTree;
import xbc.formula.engine.exp.exception.XbcExpExecException;
import xbc.formula.engine.exp.util.XbcExpDateUtil;
import xbc.formula.engine.exp.util.XbcExpNumericUtil;
import xbc.formula.engine.exp.vo.XbcExpCalculatorExp2OpVo;
import xbc.formula.engine.exp.vo.XbcExpCalculatorExpFuncDateTtcVo;
import xbc.formula.engine.exp.vo.XbcExpResultVo;

/**
 * 二元表达式执行类
 * 
 * @author leichangchun
 */
public class XbcExpCalculatorExp2Op {

	private XbcExpCalculateExp calculateExp;
	
	private int weishu;
	
	private RoundingMode roundMode;

	private String expStr;
	
	public XbcExpCalculatorExp2Op(XbcExpCalculateExp calculateExp, int weishu, RoundingMode roundMode, String expStr) {
		this.calculateExp = calculateExp;
		this.weishu = weishu;
		this.roundMode = roundMode;
		this.expStr = expStr;
	}

	protected boolean testExpVal(XbcExpExpTree exp) {
		XbcExpResultVo er = new XbcExpResultVo();
		calculateExp.calcExp(exp, er);
		if (er.value == null) {
			return false;
		}
		if (er.value.getClass() == Boolean.class || er.value.getClass() == boolean.class) {
			return (boolean) er.value;
		}
		if (er.value.getClass() == Integer.class || er.value.getClass() == int.class) {
			return (int) er.value != 0;
		}
		throw new XbcExpExecException(exp.linenum + "行: " + er.getClass().getName() + " 不能作为布尔值使用。公式: " + expStr);
	}

	protected void calc2OperandsExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpResultVo lr = new XbcExpResultVo();
		XbcExpResultVo rr = new XbcExpResultVo();
		calculateExp.calcExp(exp.lchild, lr);
		calculateExp.calcExp(exp.rchild, rr);
		if (lr.value == null) {
			lr.value = 0;
			lr.isNull = true;
		}
		if (rr.value == null) {
			rr.value = 0;
			rr.isNull = true;
		}
		switch (exp.ok) {
		case XbcExpExpOpKind.MPT_OK:
			calcMptExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.DIV_OK:
			calcDivExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.MOD_OK:
			calcModExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.PLS_OK:
			calcPlsExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.MNS_OK:
			calcMnsExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.S2L_OK:
			calcS2lExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.S2R_OK:
			calcS2rExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.BAND_OK:
			calcBandExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.BXOR_OK:
			calcBxorExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.BOR_OK:
			calcBorExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.EQ_OK:
			calcEqExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.NEQ_OK:
			calcNeqExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.LT_OK:
			calcLtExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.GT_OK:
			calcGtExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.LEQ_OK:
			calcLeqExp(lr, rr, result, exp.linenum);
			break;
		case XbcExpExpOpKind.GEQ_OK:
			calcGeqExp(lr, rr, result, exp.linenum);
			break;
		}
	}

	private static final int BOOL_TYPE_GEQ = 1;
	private static final int BOOL_TYPE_LEQ = 2;
	private static final int BOOL_TYPE_GT = 3;
	private static final int BOOL_TYPE_LT = 4;
	private static final int BOOL_TYPE_NEQ = 5;
	private static final int BOOL_TYPE_EQ = 6;

	private void calcBoolExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int type, int linenum) {
		if(isNumericType(lr.value.getClass()) && isNumericType(rr.value.getClass())) {
			if((lr.isNull && !rr.isNull) || (!lr.isNull && rr.isNull)) {
				if(type == BOOL_TYPE_EQ) {
					result.value = false;
					return;
				} else if(type == BOOL_TYPE_NEQ) {
					result.value = true;
					return;
				}
			}
			XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
			result.value = false;
			switch(type) {
				case BOOL_TYPE_GEQ:
					if(ov.lb.compareTo(ov.rb) >= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LEQ:
					if(ov.lb.compareTo(ov.rb) <= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_GT:
					if(ov.lb.compareTo(ov.rb) > 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LT:
					if(ov.lb.compareTo(ov.rb) < 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_NEQ:
					if(ov.lb.compareTo(ov.rb) != 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_EQ:
					if(ov.lb.compareTo(ov.rb) == 0) {
						result.value = true;
					}
					break;
			}
			return;
		}
		if(isNumericType(lr.value.getClass()) && rr.value.getClass() == String.class) {
			if(BOOL_TYPE_NEQ == type || BOOL_TYPE_EQ == type) {
				if(lr.isNull) {
					if(BOOL_TYPE_NEQ == type) {
						result.value = true;
					} else {
						result.value = false;
					}
					return;
				}
				BigDecimal rb = null;
				try {
					rb = new BigDecimal((String)rr.value);
				} catch(NumberFormatException nx) {
					result.value = false;
					return;
				}
				BigDecimal lb = XbcExpNumericUtil.translate2BigDecimal(lr.value, weishu, roundMode, expStr);
				result.value = false;
				if(BOOL_TYPE_NEQ == type) {
					if(lb.compareTo(rb) != 0) {
						result.value = true;
					}
				} else {
					if(lb.compareTo(rb) == 0) {
						result.value = true;
					}
				}
				return;
			}
			BigDecimal lb = XbcExpNumericUtil.translate2BigDecimal(lr.value, weishu, roundMode, expStr);
			String ls = lb.toEngineeringString();
			String rs = (String)rr.value;
			result.value = false;
			switch(type) {
				case BOOL_TYPE_GEQ:
					if(ls.compareTo(rs) >= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LEQ:
					if(ls.compareTo(rs) <= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_GT:
					if(ls.compareTo(rs) > 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LT:
					if(ls.compareTo(rs) < 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_EQ:
					break;
			}
			return;
		}
		if(lr.value.getClass() == String.class && isNumericType(rr.value.getClass())) {
			if(BOOL_TYPE_NEQ == type || BOOL_TYPE_EQ == type) {
				if(rr.isNull) {
					if(BOOL_TYPE_NEQ == type) {
						result.value = true;
					} else {
						result.value = false;
					}
					return;
				}
				BigDecimal lb = null;
				try {
					lb = new BigDecimal((String)lr.value);
				} catch(NumberFormatException nx) {
					result.value = false;
					return;
				}
				BigDecimal rb = XbcExpNumericUtil.translate2BigDecimal(rr.value, weishu, roundMode, expStr);
				result.value = false;
				if(BOOL_TYPE_NEQ == type) {
					if(lb.compareTo(rb) != 0) {
						result.value = true;
					}
				} else {
					if(lb.compareTo(rb) == 0) {
						result.value = true;
					}
				}

				return;
			}
			String ls = (String)lr.value;
			BigDecimal rb = XbcExpNumericUtil.translate2BigDecimal(rr.value, weishu, roundMode, expStr);
			String rs = rb.toEngineeringString();
			result.value = false;
			switch(type) {
				case BOOL_TYPE_GEQ:
					if(ls.compareTo(rs) >= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LEQ:
					if(ls.compareTo(rs) <= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_GT:
					if(ls.compareTo(rs) > 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LT:
					if(ls.compareTo(rs) < 0) {
						result.value = true;
					}
					break;
			}
			return;
		}
		if(lr.value.getClass() == String.class && rr.value.getClass() == String.class) {
			String ls = (String)lr.value;
			String rs = (String)rr.value;
			result.value = false;
			switch(type) {
				case BOOL_TYPE_GEQ:
					if(ls.compareTo(rs) >= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LEQ:
					if(ls.compareTo(rs) <= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_GT:
					if(ls.compareTo(rs) > 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LT:
					if(ls.compareTo(rs) < 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_NEQ:
					if(!lr.value.equals(rr.value)) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_EQ:
					if(lr.value.equals(rr.value)) {
						result.value = true;
					}
					break;
			}
			return;
		}
		if(XbcExpDateUtil.isDateType(lr.value) && XbcExpDateUtil.isDateType(rr.value) ||
				XbcExpDateUtil.isDateType(lr.value) && rr.value.getClass() == String.class ||
				lr.value.getClass() == String.class && XbcExpDateUtil.isDateType(rr.value)) {
			XbcExpCalculatorExpFuncDateTtcVo tcvl = XbcExpDateUtil.translateToCalendar("", "", lr.value);
			XbcExpCalculatorExpFuncDateTtcVo tcvr = XbcExpDateUtil.translateToCalendar("", "", rr.value);
			int rc = tcvl.getCv().compareTo(tcvr.getCv());
			result.value = false;
			switch(type) {
				case BOOL_TYPE_GEQ:
					if(rc >= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LEQ:
					if(rc <= 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_GT:
					if(rc > 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_LT:
					if(rc < 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_NEQ:
					if(rc != 0) {
						result.value = true;
					}
					break;
				case BOOL_TYPE_EQ:
					if(rc == 0) {
						result.value = true;
					}
					break;
			}
			return;
		}
		if((lr.isNull && !rr.isNull) || (!lr.isNull && rr.isNull)) {
			if(type == BOOL_TYPE_EQ) {
				result.value = false;
				return;
			} else if(type == BOOL_TYPE_NEQ) {
				result.value = true;
				return;
			}
		} else if(lr.isNull && rr.isNull) {
			if(type == BOOL_TYPE_EQ) {
				result.value = true;
				return;
			} else if(type == BOOL_TYPE_NEQ) {
				result.value = false;
				return;
			}
		}
		throw new XbcExpExecException(linenum + "行: " + lr.value.getClass() + " 与 " + rr.value.getClass() + " 不能进行对比操作。公式: " + expStr);
	}

	private void calcGeqExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		calcBoolExp(lr, rr, result, BOOL_TYPE_GEQ, linenum);
	}

	private void calcLeqExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		calcBoolExp(lr, rr, result, BOOL_TYPE_LEQ, linenum);
	}

	private void calcGtExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		calcBoolExp(lr, rr, result, BOOL_TYPE_GT, linenum);
	}

	private void calcLtExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		calcBoolExp(lr, rr, result, BOOL_TYPE_LT, linenum);
	}

	private void calcNeqExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		calcBoolExp(lr, rr, result, BOOL_TYPE_NEQ, linenum);
	}

	private void calcEqExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		calcBoolExp(lr, rr, result, BOOL_TYPE_EQ, linenum);
	}

	private void calcBorExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		int li = ov.lb.intValue();
		int ri = ov.rb.intValue();
		result.value = li | ri;
	}

	private void calcBxorExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		int li = ov.lb.intValue();
		int ri = ov.rb.intValue();
		result.value = li ^ ri;
	}

	private void calcBandExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		int li = ov.lb.intValue();
		int ri = ov.rb.intValue();
		result.value = li & ri;
	}

	private void calcS2rExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		int li = ov.lb.intValue();
		int ri = ov.rb.intValue();
		result.value = li >> ri;
	}
	
	private void calcS2lExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		int li = ov.lb.intValue();
		int ri = ov.rb.intValue();
		result.value = li << ri;
	}
	
	private void calcMnsExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		BigDecimal bv = ov.lb.subtract(ov.rb);
		result.value = bv;
	}
	
	private void calcPlsExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		if(lr.value.getClass() == String.class) {
			String rs = null;
			if(rr.value.getClass() == BigDecimal.class) {
				rs = ((BigDecimal)(rr.value)).toPlainString();
			} else {
				rs = rr.value.toString();
			}
			result.value = ((String)(lr.value)) + rs;
			return;
		}
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		BigDecimal bv = ov.lb.add(ov.rb);
		result.value = bv;
	}
	
	private void calcModExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		int li = ov.lb.intValue();
		int ri = ov.rb.intValue();
		result.value = li % ri;
	}

	private void calcDivExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		if(ov.rb.compareTo(BigDecimal.ZERO) == 0) {
			BigDecimal dbv = calculateExp.getDiv0Val();
			if(dbv == null) {
				throw new XbcExpExecException(linenum + "行: 0不能作为除数。公式： " + expStr);
			}
			ov.rb = dbv;
		}
		BigDecimal bv = ov.lb.divide(ov.rb, weishu, roundMode);
		result.value = bv;
	}

	private void calcMptExp(XbcExpResultVo lr, XbcExpResultVo rr, XbcExpResultVo result, int linenum) {
		invalidSsOperands(lr, rr, linenum);
		XbcExpCalculatorExp2OpVo ov = get2OpVo(lr, rr);
		BigDecimal bv = ov.lb.multiply(ov.rb);
		bv = bv.setScale(weishu, roundMode);
		result.value = bv;
	}
	
	private XbcExpCalculatorExp2OpVo get2OpVo(XbcExpResultVo lr, XbcExpResultVo rr) {
		XbcExpCalculatorExp2OpVo ov = new XbcExpCalculatorExp2OpVo();
		BigDecimal lb = XbcExpNumericUtil.translate2BigDecimal(lr.value, weishu, roundMode, expStr);
		BigDecimal rb = XbcExpNumericUtil.translate2BigDecimal(rr.value, weishu, roundMode, expStr);
		ov.lb = lb;
		ov.rb = rb;
		return ov;
	}
	
	private static boolean isNumericType(Class<?> clazz) {
		if (clazz != Integer.class && clazz != int.class
				&& clazz != Long.class && clazz != long.class
				&& clazz != Double.class && clazz != double.class
				&& clazz != BigDecimal.class) {
			return false;
		}
		return true;
	}

	private void invalidSsOperands(XbcExpResultVo lr, XbcExpResultVo rr, int linenum) {
		if (!isNumericType(lr.value.getClass())) {
			throw new XbcExpExecException(linenum + "行: 运算不支持的数据类型: " + lr.value.getClass().getName() + "。公式: " + expStr);
		}
		if (!isNumericType(rr.value.getClass())) {
			throw new XbcExpExecException(linenum + "行: 运算不支持的数据类型: " + rr.value.getClass().getName() + "。公式: " + expStr);
		}
	}
	
}
