package xbc.formula.engine.exp.exec;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import xbc.formula.engine.exp.datatype.XbcExpExpKind;
import xbc.formula.engine.exp.datatype.XbcExpExpOpKind;
import xbc.formula.engine.exp.datatype.XbcExpExpTree;
import xbc.formula.engine.exp.exception.XbcExpExecException;
import xbc.formula.engine.exp.ext.XbcExpExtFuncManager;
import xbc.formula.engine.exp.util.XbcExpNumericUtil;
import xbc.formula.engine.exp.vo.XbcExpResultVo;

/**
 * 表达式计算引擎自带函数执行类
 * 
 * @author leichangchun
 */
public class XbcExpCalculatorExpFunc {

	private XbcExpCalculateExp calculateExp;

	private int weishu;
	
	private RoundingMode roundMode;

	private String expStr;
	
	private Object[] formulaParams;
	
	private XbcExpCalculatorExpFuncRound calculatorExpFuncRound;
	
	private XbcExpCalculatorExpFuncDate calculatorExpFuncDate;
	
	public XbcExpCalculatorExpFunc(XbcExpCalculateExp calculateExp, int weishu, RoundingMode roundMode, String expStr, Object[] formulaParams) {
		this.calculateExp = calculateExp;
		this.weishu = weishu;
		this.roundMode = roundMode;
		this.expStr = expStr;
		this.formulaParams = formulaParams;
		calculatorExpFuncRound = new XbcExpCalculatorExpFuncRound(this, weishu, roundMode, expStr);
		calculatorExpFuncDate = new XbcExpCalculatorExpFuncDate(this, expStr);
	}

	protected Method findMethod(Class<?> clazz, String methodName, List<Object> pvs) {
		boolean hasNullParam = false;
		Class<?>[] pts = new Class<?>[pvs.size()];
		for(int k = 0;k < pvs.size();k ++) {
			if(pvs.get(k) == null) {
				hasNullParam = true;
				break;
			} else {
				pts[k] = pvs.get(k).getClass();
			}
		}
		if(!hasNullParam) {
			Method method = null;
			try {
				method = clazz.getDeclaredMethod(methodName, pts);
			} catch (NoSuchMethodException | SecurityException e) {
				//throw new XbcExpExecException("获取方法" + methodName + "失败。公式: " + expStr);
			}
			if(method != null) {
				return method;
			}
		}
		Method[] methods = clazz.getMethods();
		for(Method method : methods) {
			if(method.getParameterCount() == pvs.size() && method.getName().equals(methodName)) {
				if(!hasNullParam) {
					Class<?>[] pts2 = method.getParameterTypes();
					boolean matched = true;
					for(int k = 0;k < pts.length;k ++) {
						Class<?> c1 = translateClass(pts[k]);
						Class<?> c2 = translateClass(pts2[k]);
						if(!c2.isAssignableFrom(c1)) {
							matched = false;
							break;
						}
					}
					if(matched) {
						return method;
					}
				} else {
					return method;
				}
			}
		}
		return null;
	}
	
	private Class<?> translateClass(Class<?> c) {
		if(c == int.class) {
			return Integer.class;
		}
		if(c == long.class) {
			return Long.class;
		}
		if(c == double.class) {
			return Double.class;
		}
		if(c == float.class) {
			return Float.class;
		}
		if(c == boolean.class) {
			return Boolean.class;
		}
		if(c == short.class) {
			return Short.class;
		}
		if(c == byte.class) {
			return Byte.class;
		}
		return c;
	}
	
	private void calcPrdExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpExpTree lchild = exp.lchild;
		String methodName = (String)lchild.value;
		List<Object> pvs = getFuncParams(exp);
		XbcExpResultVo pr = new XbcExpResultVo();
		calculateExp.calcExp(lchild.lchild, pr);
		if(pr.value == null) {
			throw new XbcExpExecException(exp.linenum + "行: " + methodName + " 的前面为空值。公式: " + expStr);
		}
		Method method = findMethod(pr.value.getClass(), methodName, pvs);
		if(method == null) {
			throw new XbcExpExecException(exp.linenum + "行: " + pr.value.getClass().getName() + " 里面，没有找到参数个数为 " + pvs.size() + "个的名称为" + methodName + "的方法。公式: " + expStr);
		}
		Object[] args = new Object[pvs.size()];
		for(int k = 0;k < pvs.size();k ++) {
			args[k] = pvs.get(k);
		}
		try {
			Object ret = method.invoke(pr.value, args);
			result.value = ret;
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			throw new XbcExpExecException(e, exp.linenum + "行: " + methodName + "执行出错。");
		}
	}

	public void calcFuncExp(XbcExpExpTree exp, XbcExpResultVo result) {
		XbcExpExpTree lchild = exp.lchild;
		if(lchild.ek == XbcExpExpKind.OP_EK && lchild.ok == XbcExpExpOpKind.PRD_OK) {
			calcPrdExp(exp, result);
			return;
		}
		String funcName = (String)exp.lchild.value;
		if(funcName.equals("min")) {
			calcMinFunc(exp, result);
		} else if(funcName.equals("max")) {
			calcMaxFunc(exp, result);
		} else if(funcName.equals("write")) {
			calcWriteFunc(exp, result);
		} else if(funcName.equals("writeln")) {
			calcWritelnFunc(exp, result);
		} else if(funcName.equals("roundUp")) {
			calculatorExpFuncRound.calcRoundUpFunc(exp, result);
		} else if(funcName.equals("roundHalfUp")) {
			calculatorExpFuncRound.calcRoundHalfUpFunc(exp, result);
		} else if(funcName.equals("roundDown")) {
			calculatorExpFuncRound.calcRoundDownFunc(exp, result);
		} else if(funcName.equals("compareDate")) {
			calculatorExpFuncDate.calcCompareDateFunc(exp, result);
		} else if(funcName.equals("addDate")) {
			calculatorExpFuncDate.calcAddDateFunc(exp, result);
		} else if(funcName.equals("addMonth")) {
			calculatorExpFuncDate.calcAddMonthFunc(exp, result);
		} else if(funcName.equals("addHour")) {
			calculatorExpFuncDate.calcAddHourFunc(exp, result);
		} else if(funcName.equals("dateToString")) {
			calculatorExpFuncDate.calcDateToStringFunc(exp, result);
		} else if(funcName.equals("thisDay") || funcName.equals("today")) {
			calculatorExpFuncDate.calcThisDayFunc(exp, result);
		} else if(funcName.equals("now")) {
			calculatorExpFuncDate.calcNowFunc(exp, result);
		} else if(funcName.equals("diffDate")) {
			calculatorExpFuncDate.calcDiffDateFunc(exp, result);
		} else if(funcName.equals("toDate")) {
			calculatorExpFuncDate.toDateFunc(exp, result);
		} else {
			List<Object> pvs = getFuncParams(exp);
			Object bv = XbcExpExtFuncManager.executeFunc(funcName, pvs, expStr, formulaParams);
			result.value = bv;
		}
	}
	
	protected List<Object> getFuncParams(XbcExpExpTree exp) {
		List<Object> pvs = new ArrayList<>();
		XbcExpResultVo pr = new XbcExpResultVo();
		for(XbcExpExpTree pe : exp.params) {
			calculateExp.calcExp(pe, pr);
			pvs.add(pr.value);
		}
		return pvs;
	}
	
	private void calcMinFunc(XbcExpExpTree exp, XbcExpResultVo result) {
		if(exp.params.size() < 1) {
			throw new XbcExpExecException(exp.linenum + "行: min函数至少需要一个参数。");
		}
		List<Object> pvs = getFuncParams(exp);
		BigDecimal minBv = compMaxMinVal(pvs, false);
		result.value = minBv;
	}
	
	private void calcMaxFunc(XbcExpExpTree exp, XbcExpResultVo result) {
		if(exp.params.size() < 1) {
			throw new XbcExpExecException(exp.linenum + "行: max函数至少需要一个参数。");
		}
		List<Object> pvs = getFuncParams(exp);
		BigDecimal maxBv = compMaxMinVal(pvs, true);
		result.value = maxBv;
	}

	@SuppressWarnings("unchecked")
	private BigDecimal compMaxMinVal(List<Object> pvs, boolean isMax) {
		BigDecimal mv = null;
		Object ov = pvs.get(0);
		if(ov != null && List.class.isAssignableFrom(ov.getClass())) {
			mv = compMaxMinVal((List<Object>)ov, isMax);
		} else {
			mv = XbcExpNumericUtil.translate2BigDecimal(ov, weishu, roundMode, expStr);
		}
		for(int k = 1;k < pvs.size();k ++) {
			BigDecimal bv = null;
			ov = pvs.get(k);
			if(ov != null && List.class.isAssignableFrom(ov.getClass())) {
				bv = compMaxMinVal((List<Object>)ov, isMax);
			} else {
				bv = XbcExpNumericUtil.translate2BigDecimal(ov, weishu, roundMode, expStr);
			}
			if(isMax) {
				if(mv.compareTo(bv) < 0) {
					mv = bv;
				}
			} else {
				if(mv.compareTo(bv) > 0) {
					mv = bv;
				}
			}
		}
		return mv;
	}
	
	private void calcWriteFunc(XbcExpExpTree exp, XbcExpResultVo result) {
		if(exp.params.size() < 1) {
			throw new XbcExpExecException(exp.linenum + "行: write函数至少需要一个参数。");
		}
		List<Object> pvs = getFuncParams(exp);
		for(int k = 0;k < pvs.size();k ++) {
			if(pvs.get(k) != null) {
				System.out.print(pvs.get(k).toString());
			} else {
				System.out.print("null");
			}
		}
	}
	
	private void calcWritelnFunc(XbcExpExpTree exp, XbcExpResultVo result) {
		if(exp.params.size() >= 1) {
			calcWriteFunc(exp, result);
		}
		System.out.println();
	}
}
