package xbc.formula.engine.exp.exec;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

import xbc.formula.engine.exp.datatype.XbcExpExpTree;
import xbc.formula.engine.exp.exception.XbcExpExecException;
import xbc.formula.engine.exp.util.XbcExpNumericUtil;
import xbc.formula.engine.exp.vo.XbcExpResultVo;

/**
 * 表达式计算引擎自带函数(取舍类函数)执行类
 * 
 * @author leichangchun
 */
public class XbcExpCalculatorExpFuncRound {

	private XbcExpCalculatorExpFunc calculatorExpFunc;

	private int weishu;
	
	private RoundingMode roundMode;

	private String expStr;
	
	public XbcExpCalculatorExpFuncRound(XbcExpCalculatorExpFunc calculatorExpFunc, int weishu, RoundingMode roundMode, String expStr) {
		this.calculatorExpFunc = calculatorExpFunc;
		this.weishu = weishu;
		this.roundMode = roundMode;
		this.expStr = expStr;
	}

    protected void calcRoundUpFunc(XbcExpExpTree exp, XbcExpResultVo result) {
        calcRoundFunc("roundUp", RoundingMode.UP, exp, result);
    }

    protected void calcRoundHalfUpFunc(XbcExpExpTree exp, XbcExpResultVo result) {
        calcRoundFunc("roundHalfUp", RoundingMode.HALF_UP, exp, result);
    }

    protected void calcRoundDownFunc(XbcExpExpTree exp, XbcExpResultVo result) {
        calcRoundFunc("roundDown", RoundingMode.DOWN, exp, result);
    }

    private void calcRoundFunc(String funcName, RoundingMode roundMode, XbcExpExpTree exp, XbcExpResultVo result) {
        if(exp.params.size() < 2) {
            throw new XbcExpExecException(exp.linenum + "行: " + funcName + "函数需要两个参数。");
        }
        List<Object> pvs = calculatorExpFunc.getFuncParams(exp);
        if(pvs.get(0) == null || pvs.get(1) == null) {
            throw new XbcExpExecException(exp.linenum + "行: " + funcName + "函数的第一个参数和第二个参数都不能为空。");
        }
        BigDecimal bv1 = getBigDecimalParam(funcName, "一", pvs.get(0), exp.linenum);
        BigDecimal bv2 = getBigDecimalParam(funcName, "二", pvs.get(1), exp.linenum);
        double scale = bv2.doubleValue();
        if(scale <= 0) {
            throw new XbcExpExecException(exp.linenum + "行: " + funcName + "函数的第二个参数必须大于0");
        }
        BigDecimal bv = round(bv1, scale, roundMode);
        result.value = bv;
    }

    private static BigDecimal round(BigDecimal bv, double scale, RoundingMode roundMode) {
        int sc = 0;
        if(scale < 1) {
            while(scale < 1) {
                sc ++;
                scale *= 10;
            }
        } else if(scale >= 1 && scale < 10) {
            sc = 0;
        } else {
            while(scale > 1) {
                sc --;
                scale /= 10;
            }
        }
        BigDecimal bt = bv.setScale(sc, roundMode);
        bt = new BigDecimal(bt.toPlainString());
        return bt;
    }
    
    private BigDecimal getBigDecimalParam(String funcName, String pidx, Object ov, int linenum) {
        BigDecimal bv = null;
        try {
            bv = XbcExpNumericUtil.translate2BigDecimal(ov, weishu, roundMode, expStr);
        } catch(XbcExpExecException x) {
            throw new XbcExpExecException(linenum + "行: " + funcName + "函数的第" + pidx + "个参数不是数值类型的参数。");
        }
        return bv;
    }
}
