package xbc.formula.engine.exp.exec;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import xbc.formula.engine.exp.datatype.XbcExpExpTree;
import xbc.formula.engine.exp.exception.XbcExpExecException;
import xbc.formula.engine.exp.vo.XbcExpResultVo;

public class XbcExpCalculatorExpNew {
	
	private XbcExpCalculatorExpFunc calculatorExpFunc;

	private String expStr;

	public XbcExpCalculatorExpNew(XbcExpCalculatorExpFunc calculatorExpFunc, String expStr) {
		this.calculatorExpFunc = calculatorExpFunc;
		this.expStr = expStr;
	}

	public void calcInvokeExp(XbcExpExpTree exp, XbcExpResultVo result) {
		List<Object> pvs = calculatorExpFunc.getFuncParams(exp);
		if(pvs == null || pvs.size() < 2) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数至少需要两个参数: 类名， 方法名。公式: " + expStr);
		}
		Object p2 = pvs.get(1);
		if(p2 == null) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数的第二个参数不能为null。公式: " + expStr);
		}
		if(p2.getClass() != String.class) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数的第二个参数必须为字符串。公式: " + expStr);
		}
		Class<?> clazz = getClass4Create(exp, pvs);
		String methodName = (String)p2;
		List<Object> mpvs = new ArrayList<>();
		for(int k = 2;k < pvs.size();k ++) {
			mpvs.add(pvs.get(k));
		}
		Method method = calculatorExpFunc.findMethod(clazz, methodName, mpvs);
		if(method == null) {
			throw new XbcExpExecException(exp.linenum + "行: " + clazz.getName() + " 里面，没有找到参数个数为 " + mpvs.size() + "个的名称为" + methodName + "的方法。公式: " + expStr);
		}
		Object[] args = new Object[mpvs.size()];
		for(int k = 0;k < mpvs.size();k ++) {
			args[k] = mpvs.get(k);
		}
		try {
			Object ret = method.invoke(null, args);
			result.value = ret;
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			e.printStackTrace();
			throw new XbcExpExecException(e, exp.linenum + "行: " + methodName + "执行出错。");
		}
	}

	public void calcNewExp(XbcExpExpTree exp, XbcExpResultVo result) {
		List<Object> pvs = calculatorExpFunc.getFuncParams(exp);
		if(pvs == null || pvs.size() == 0) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数必须要跟参数。公式: " + expStr);
		}
		Class<?> clazz = getClass4Create(exp, pvs);
		Constructor<?> cc = getConstructor4Class(exp, clazz, pvs);
		Object[] pts = new Object[pvs.size() - 1];
		for(int k = 1;k < pvs.size();k ++) {
			pts[k - 1] = pvs.get(k);
		}
		try {
			Object inst = cc.newInstance(pts);
			result.value = inst;
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException
				| InvocationTargetException e) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数运行失败，实例化类[" + (String)(pvs.get(0)) + "]失败。公式: " + expStr);
		}
	}

	private Class<?> getClass4Create(XbcExpExpTree exp, List<Object> pvs) {
		Object p1 = pvs.get(0);
		if(p1 == null) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数的第一个参数不能为null。公式: " + expStr);
		}
		Class<?> clazz = p1.getClass();
		if(clazz != Class.class && clazz != String.class) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数的第一个参数必须是 Class 类型或 String 类型。公式: " + expStr);
		}
		if(clazz == String.class) {
			try {
				clazz = Class.forName((String)p1);
			} catch (ClassNotFoundException e) {
				throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数运行失败，类[" + (String)p1 + "]无法被加载。公式: " + expStr);
			}
		}
		return clazz;
	}
	
	private Constructor<?> getConstructor4Class(XbcExpExpTree exp, Class<?> clazz, List<Object> pvs) {
		Object p1 = pvs.get(0);
		Constructor<?>[] cs = clazz.getConstructors();
		if(cs == null || cs.length == 0) {
			throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数运行失败，类[" + (String)p1 + "]没有构造函数。公式: " + expStr);
		}
		int pc = pvs.size() - 1;
		boolean dup = false;
		Constructor<?> cc = null;
		for(Constructor<?> c : cs) {
			if(c.getParameterCount() == pc) {
				if(cc != null) {
					dup = true;
				} else {
					cc = c;
				}
			}
		}
		if(dup) {
			boolean hasNullParam = false;
			Class<?>[] pts = new Class<?>[pvs.size() - 1];
			for(int k = 1;k < pvs.size();k ++) {
				if(pvs.get(k) == null) {
					hasNullParam = true;
					break;
				} else {
					pts[k - 1] = pvs.get(k).getClass();
				}
			}
			if(hasNullParam) {
				throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数运行失败，无法匹配到类[" + (String)p1 + "]的构造函数。公式: " + expStr);
			}
			try {
				cc = clazz.getConstructor(pts);
			} catch (NoSuchMethodException | SecurityException e) {
				throw new XbcExpExecException(exp.linenum + "行: " + (String)exp.value + "函数运行失败，获取类[" + (String)p1 + "]的构造函数失败。公式: " + expStr);
			}
		}
		return cc;
	}

}
