package xbc.formula.engine.exp.ext;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import xbc.formula.engine.exp.exception.XbcExpExecException;

/**
 * 扩展函数管理工具
 * 
 * @author leichangchun
 */
public class XbcExpExtFuncManager {

	private static Map<String, XbcExpExtFunc> exFuncMap = new HashMap<>();
	
	/**
	 * 向计算工具注册扩展函数
	 * @param funcName 扩展函数名称 
	 * @param func 扩展函数的实现类
	 */
	public synchronized static void registerFunc(String funcName, XbcExpExtFunc func) {
		if(exFuncMap.containsKey(funcName)) {
			return;
		}
		exFuncMap.put(funcName, func);
	}
	
	public static XbcExpExtFunc getExtFunc(String funcName) {
		return exFuncMap.get(funcName);
	}
	
	public static Object executeFunc(String funcName, List<Object> pvs, String expStr, Object[] formulaParams) {
		XbcExpExtFunc func = XbcExpExtFuncManager.getExtFunc(funcName);
		if(func == null) {
			throw new XbcExpExecException("不支持的函数: " + funcName + "。");
		} else {
			Object bv = func.calculate(pvs, formulaParams);
			return bv;
		}
	}
}
