package xbc.formula.engine.exp.util;

import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import xbc.formula.engine.exp.exception.XbcExpParamException;
import xbc.formula.engine.exp.vo.XbcExpCalculatorExpFuncDateTtcVo;

public class XbcExpDateUtil {

    /**
     * 将字符串转换成Date
     *
     * @param dateStr
     * @return 失败时返回null。
     */
    public static Date dateFromString(String dateStr) {
        if (dateStr == null || dateStr.isEmpty()) {
            return null;
        }
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        try {
            return new Date(formatter.parse(dateStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        formatter = new SimpleDateFormat("yyyy/MM/dd");
        try {
            return new Date(formatter.parse(dateStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        return null;
    }

    /**
     * 将字符串转换成Timestamp
     *
     * @param tsStr
     * @return 失败时返回null。
     */
    public static Timestamp timestampFromString(String tsStr) {
        if (tsStr == null || tsStr.isEmpty()) {
            return null;
        }
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        try {
            return new Timestamp(formatter.parse(tsStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            return new Timestamp(formatter.parse(tsStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        formatter = new SimpleDateFormat("yyyy-MM-dd");
        try {
            return new Timestamp(formatter.parse(tsStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.SSS");
        try {
            return new Timestamp(formatter.parse(tsStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        try {
            return new Timestamp(formatter.parse(tsStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        formatter = new SimpleDateFormat("yyyy/MM/dd");
        try {
            return new Timestamp(formatter.parse(tsStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        formatter = new SimpleDateFormat("HH:mm:ss");
        try {
            return new Timestamp(formatter.parse(tsStr).getTime());
        } catch (ParseException e) {
            // e.printStackTrace();
        }
        return null;
    }

    public static String thisDay() {
        Calendar c = Calendar.getInstance();
        Timestamp tv = new Timestamp(c.getTimeInMillis());
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String ds = formatter.format(tv);
        return ds;
    }

    public static String now() {
        Calendar c = Calendar.getInstance();
        Timestamp tv = new Timestamp(c.getTimeInMillis());
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        String ds = formatter.format(tv);
        return ds;
    }

    public static String calendarToString(Calendar c, String format) {
        Timestamp tv = new Timestamp(c.getTimeInMillis());
        SimpleDateFormat formatter = new SimpleDateFormat(format);
        String ds = formatter.format(tv);
        return ds;
    }

    public static int diffDate(Calendar c1, Calendar c2) {
        // 将时分秒毫秒 设置为0
        c1.set(Calendar.HOUR_OF_DAY, 0);
        c1.set(Calendar.MINUTE, 0);
        c1.set(Calendar.SECOND, 0);
        c1.set(Calendar.MILLISECOND, 0);
        c2.set(Calendar.HOUR_OF_DAY, 0);
        c2.set(Calendar.MINUTE, 0);
        c2.set(Calendar.SECOND, 0);
        c2.set(Calendar.MILLISECOND, 0);
        long dl = c1.getTimeInMillis() - c2.getTimeInMillis();
        int iv = (int) (dl / (1000 * 60 * 60 * 24));
        return iv;
    }

    public static Calendar calendarFromString(String ds) {
        Date dv = dateFromString(ds);
        Calendar c = Calendar.getInstance();
        c.setTime(dv);
        return c;
    }

    public static boolean isDateType(Object ov) {
        if(ov == null) {
            return false;
        }
        Class<?> clazz = ov.getClass();
        if(clazz == java.util.Date.class || clazz == java.sql.Date.class || clazz == Timestamp.class || Calendar.class.isAssignableFrom(clazz)) {
            return true;
        }
        if(clazz == String.class) {
            Timestamp tv = timestampFromString((String) ov);
            if (tv == null) {
                java.sql.Date dv = dateFromString((String) ov);
                if (dv == null) {
                    return false;
                }
                return true;
            } else {
                return true;
            }
        }
        return false;
    }

    public static XbcExpCalculatorExpFuncDateTtcVo translateToCalendar(String funcName, String pidx, Object ov) {
        Calendar c = Calendar.getInstance();
        Class<?> clazz = ov.getClass();
        if(clazz == java.util.Date.class) {
            java.util.Date dv = (java.util.Date)ov;
            c.setTimeInMillis(dv.getTime());
        } else if(clazz == java.sql.Date.class) {
            java.sql.Date dv = (java.sql.Date)ov;
            c.setTimeInMillis(dv.getTime());
        } else if(clazz == Timestamp.class) {
            Timestamp tv = (Timestamp)ov;
            c.setTimeInMillis(tv.getTime());
        } else if(Calendar.class.isAssignableFrom(clazz)) {
            Calendar cv = (Calendar) ov;
            c.setTimeInMillis(cv.getTimeInMillis());
        } else if(clazz == String.class) {
            Timestamp tv = timestampFromString((String)ov);
            if(tv == null) {
                java.sql.Date dv = dateFromString((String)ov);
                if(dv == null) {
                    throw new XbcExpParamException(funcName + "函数的第" + pidx + "个参数无法转换成日期类型: " + (String)ov);
                }
                clazz = java.sql.Date.class;
                c.setTimeInMillis(dv.getTime());
            } else {
                clazz = Timestamp.class;
                c.setTimeInMillis(tv.getTime());
            }
        } else {
            throw new XbcExpParamException(funcName + "函数的第" + pidx + "个参数必须为日期类型的参数。");
        }
        XbcExpCalculatorExpFuncDateTtcVo ttcVo = new XbcExpCalculatorExpFuncDateTtcVo();
        ttcVo.setClazz(clazz);
        ttcVo.setCv(c);
        return ttcVo;
    }
}
