package xbc.formula.calcmgr.logic.impl;

import java.sql.Timestamp;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import xbc.formula.calcmgr.dao.CalcNoManagerDao;
import xbc.formula.calcmgr.logic.CalcNoManagerLogic;
import xbc.formula.calcmgr.vo.CalcNoVo;

@Component
public class CalcNoManagerLogicImpl implements CalcNoManagerLogic {

	private static final int NO_TYPE_FORMULA = 1;
	
	@Autowired
	private CalcNoManagerDao calcNoManagerDao;
	
	/**
	 * 此处通过锁表的行数据来实现id发行，并确保系统内唯一。
	 * 为了避免锁行时间过长，所以，放在单独的事务里面进行。 
	 */
	@Override
	@Transactional(propagation= Propagation.REQUIRES_NEW)
	public CalcNoVo getResultId(Timestamp ts, int isize, int rsize) {
		CalcNoVo cnv = calcNoManagerDao.getCurrentNoLock(NO_TYPE_FORMULA);
		long baseFormulaId = cnv.getFormulaNo();
		long baseIndicatorId = cnv.getIndicatorNo();
		long baseRuleId = cnv.getRuleNo();
		cnv.setFormulaNo(baseFormulaId + 1);
		cnv.setIndicatorNo(baseIndicatorId + isize);
		cnv.setRuleNo(baseRuleId + rsize);
		cnv.setModifyTime(ts);
		cnv.setResultType(NO_TYPE_FORMULA);
		calcNoManagerDao.updateCurrentNo(cnv);
		cnv.setFormulaNo(baseFormulaId);
		cnv.setIndicatorNo(baseIndicatorId);
		cnv.setRuleNo(baseRuleId);
		return cnv;
	}

}
