package xbc.formula.calcmgr.logic.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import xbc.formula.calcmgr.dao.CalcFormulaDao;
import xbc.formula.calcmgr.logic.CalculateManagerLogic;
import xbc.formula.calcmgr.logic.SaveResultLogic;
import xbc.formula.calcmgr.vo.CalculateRequestVo;
import xbc.formula.calcmgr.vo.CalculateResponseVo;
import xbc.formula.engine.exp.vo.XbcExpRuleVo;
import xbc.formula.engine.fml.XbcFmlCalculator;
import xbc.formula.engine.fml.vo.XbcFmlFormulaVo;
import xbc.formula.fmlmgr.logic.FormulaLogic;
import xbc.formula.framework.exception.FormulaException;

@Component
public class CalculateManagerLogicImpl implements CalculateManagerLogic {
	
	@Autowired
	private CalcFormulaDao calcFormulaDao;
	
	@Autowired
	private FormulaLogic formulaLogic;
	
	@Autowired
	private SaveResultLogic saveResultLogic;

	@Override
	public CalculateResponseVo calculate(CalculateRequestVo requestVo) {
		if(requestVo == null) {
			throw new FormulaException("请输入计算参数。");
		}
		String formulaCode = calcFormulaDao.getFormulaCode(requestVo.getProductId(), requestVo.getBizType());
		if(formulaCode == null) {
			throw new FormulaException("没有找到对应的公式编码，product_id: " + requestVo.getProductId() + ", biz_type: " + requestVo.getBizType());
		}
		XbcFmlFormulaVo fv = formulaLogic.getFormulaByCode(formulaCode);
		if(fv == null) {
			throw new FormulaException("没有找到公式，公式编码: " + formulaCode);
		}
		List<XbcExpRuleVo> ruleVos = new ArrayList<>();
		Object ov = XbcFmlCalculator.calculte(fv, ruleVos, requestVo.getParamMap());
		CalculateResponseVo responseVo = new CalculateResponseVo();
		responseVo.setResultType(ov.getClass().getName());
		responseVo.setResultValue(ov.toString());
		saveResultLogic.saveResult(requestVo, responseVo, ruleVos, fv);
		return responseVo;
	}

}
