package xbc.formula.calcmgr.logic.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import xbc.formula.calcmgr.dao.CalcIndicatorResultDao;
import xbc.formula.calcmgr.dao.CalcResultDao;
import xbc.formula.calcmgr.dao.entity.CalcIndicatorResult;
import xbc.formula.calcmgr.dao.entity.CalcResult;
import xbc.formula.calcmgr.logic.QueryCalcResultLogic;
import xbc.formula.calcmgr.vo.CalcResultDescVo;
import xbc.formula.engine.fml.vo.XbcFmlFormulaIndicatorVo;
import xbc.formula.framework.exception.FormulaException;

@Component
public class QueryCalcResultLogicImpl implements QueryCalcResultLogic {
	
	@Autowired
	private CalcResultDao calcResultDao;
	
	@Autowired
	private CalcIndicatorResultDao calcIndicatorResultDao;

	@Override
	public CalcResultDescVo getCalcResultDesc(long calcResultId) {
		CalcResult cr = calcResultDao.getByKeyId(calcResultId);
		if(cr == null) {
			throw new FormulaException("没有找到计算结果, keyId: " + calcResultId);
		}
		CalcResultDescVo crdv = new CalcResultDescVo();
		crdv.setProductId(cr.getProductId());
		crdv.setBizType(cr.getBizType());
		crdv.setBizCode(cr.getBizCode());
		crdv.setFormulaName(cr.getFormulaName());
		crdv.setFormulaCode(cr.getFormulaCode());
		crdv.setFormulaValue(cr.getFormulaValue());
		List<CalcIndicatorResult> cirs = calcIndicatorResultDao.getByCalcResultId(calcResultId);
        StringBuilder sb = new StringBuilder();
		sb.append(cr.getFormulaName() + ": " + cr.getCalcResult() + ": " + cr.getFormulaValue() + "\r\n");
		for(CalcIndicatorResult cir : cirs) {
			String ssv = null;
        	if(cir.getIndicatorType() == XbcFmlFormulaIndicatorVo.PARAM_TYPE) {
        		ssv = "(" + cir.getSpecifiedVal() + ")";
        	} else {
        		ssv = "";
        	}
            String line = getPrefix(cir.getNodeCode()) + cir.getIndicatorName() + ssv + ": " + cir.getCalcResult();
            sb.append(line);
            if(cir.getIndicatorType() == XbcFmlFormulaIndicatorVo.FORMULA_TYPE) {
                sb.append(": " + cir.getFormulaValue());
            }
            sb.append("\r\n");
		}
        String desc = sb.toString();
        crdv.setDesc(desc);
		return crdv;
	}

    private String getPrefix(String nodeCode) {
        StringBuilder sb = new StringBuilder();
        for(int k = 0;k < nodeCode.length();k ++) {
            sb.append(' ');
        }
        return sb.toString();
    }
}
