package xbc.formula.calcmgr.logic.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import xbc.formula.calcmgr.dao.CalcIndicatorResultDao;
import xbc.formula.calcmgr.dao.CalcResultDao;
import xbc.formula.calcmgr.dao.CalcRuleResultDao;
import xbc.formula.calcmgr.dao.CalcRuleResultParamDao;
import xbc.formula.calcmgr.dao.entity.CalcIndicatorResult;
import xbc.formula.calcmgr.dao.entity.CalcResult;
import xbc.formula.calcmgr.dao.entity.CalcRuleResult;
import xbc.formula.calcmgr.dao.entity.CalcRuleResultParam;
import xbc.formula.calcmgr.logic.CalcNoManagerLogic;
import xbc.formula.calcmgr.logic.SaveResultLogic;
import xbc.formula.calcmgr.vo.CalcNoVo;
import xbc.formula.calcmgr.vo.CalculateRequestVo;
import xbc.formula.calcmgr.vo.CalculateResponseVo;
import xbc.formula.engine.exp.vo.XbcExpRuleParamVo;
import xbc.formula.engine.exp.vo.XbcExpRuleVo;
import xbc.formula.engine.fml.vo.XbcFmlFormulaIndicatorVo;
import xbc.formula.engine.fml.vo.XbcFmlFormulaVo;
import xbc.formula.framework.exception.FormulaException;

@Component
public class SaveResultLogicImpl implements SaveResultLogic {
	
	@Autowired
	private CalcNoManagerLogic calcNoManagerLogic;
	
	@Autowired
	private CalcResultDao calcResultDao;
	
	@Autowired
	private CalcIndicatorResultDao calcIndicatorResultDao;
	
	@Autowired
	private CalcRuleResultDao calcRuleResultDao;
	
	@Autowired
	private CalcRuleResultParamDao calcRuleResultParamDao;
	
	private Method calcIndicatorResultDaoMethod;
	private Method calcRuleResultDaoMethod;
	private Method calcRuleResultParamDaoMethod;

	private Method findMethod(Object dao) {
		Method method = null;
		Method[] ms = dao.getClass().getDeclaredMethods();
		for (Method m : ms) {
			if (m.getName().equals("saveBatch")) {
				method = m;
				break;
			}
		}
		if (method == null) {
			throw new FormulaException("系统异常。"); // 不会出现这种情况
		}
		return method;
	}

	@PostConstruct
	private void initMethods() {
		calcIndicatorResultDaoMethod = findMethod(calcIndicatorResultDao);
		calcRuleResultDaoMethod = findMethod(calcRuleResultDao);
		calcRuleResultParamDaoMethod = findMethod(calcRuleResultParamDao);
	}
	
	@Override
	public void saveResult(CalculateRequestVo requestVo, CalculateResponseVo responseVo, List<XbcExpRuleVo> ruleVos, XbcFmlFormulaVo fv) {
		if(!requestVo.isSaveResult()) {
			responseVo.setCalcResultId(0);
			return;
		}
		
		Timestamp thistime = new Timestamp(System.currentTimeMillis());
		CalcResult cr = createFormulaResult(requestVo, fv, thistime);
		RetrieveIndicatorVo riv = new RetrieveIndicatorVo();
		riv.thistime = thistime;
		riv.cirs = new ArrayList<>();
		riv.savedFvSet = new HashSet<>();
		riv.pos = 1;
		retrieveIndicators(fv, 0, "", riv);
		
		CalcNoVo cnv = calcNoManagerLogic.getResultId(thistime, riv.cirs.size(), ruleVos == null ? 0 : ruleVos.size());
		long formulaResultId = cnv.getFormulaNo() + 1;
		cr.setKeyId(formulaResultId);
		if(riv.cirs.size() > 0) {
			long baseIndicatorId = cnv.getIndicatorNo();
			for(CalcIndicatorResult cir : riv.cirs) {
				cir.setCalcResultId(formulaResultId);
				cir.setKeyId(cir.getKeyId() + baseIndicatorId);
				long pid = cir.getParentId();
				if(pid > 0) {
					cir.setParentId(pid + baseIndicatorId);
				}
			}
			saveBatch(calcIndicatorResultDao, calcIndicatorResultDaoMethod, riv.cirs);
		}
		if(ruleVos != null && ruleVos.size() > 0) {
			long baseRuleId = cnv.getRuleNo();
			List<CalcRuleResult> crrvs = new ArrayList<>();
			List<CalcRuleResultParam> crrps = new ArrayList<>();
			for(int k = 0;k < ruleVos.size();k ++) {
				XbcExpRuleVo rv = ruleVos.get(k);
				CalcRuleResult crrv = new CalcRuleResult();
				long ruleResultId = baseRuleId + k + 1;
				crrv.setKeyId(ruleResultId);
				crrv.setCalcResultId(formulaResultId);
				crrv.setRuleType(rv.getType() == null ? "" : rv.getType());
				crrv.setRuleName(rv.getName() == null ? "" : rv.getName());
				crrv.setBizIdCode(rv.getBizIdCode() == null ? "" : rv.getBizIdCode());
				crrv.setDisplayOrder(rv.getDisplayOrder());
				crrv.setCalcResult(objToString(rv.getValue()));
				crrv.setRuleDesc(rv.getDesc() == null ? "" : rv.getDesc());
				crrv.setComment(rv.getComment() == null ? "" : rv.getComment());
				crrv.setCreateTime(thistime);
				crrvs.add(crrv);
				List<XbcExpRuleParamVo> rpvs = rv.getRpvs();
				if(rpvs != null && rpvs.size() > 0) {
					for(int j = 0;j < rpvs.size();j ++) {
						XbcExpRuleParamVo rpv = rpvs.get(j);
						CalcRuleResultParam crrp = new CalcRuleResultParam();
						crrp.setRuleResultId(ruleResultId);
						crrp.setDisplayOrder(j + 1);
						crrp.setParamName(rpv.getPname());
						crrp.setParamValue(objToString(rpv.getValue()));
						crrp.setCreateTime(thistime);
						crrps.add(crrp);
					}
				}
			}
			saveBatch(calcRuleResultDao, calcRuleResultDaoMethod, crrvs);
			if(crrps.size() > 0) {
				saveBatch(calcRuleResultParamDao, calcRuleResultParamDaoMethod, crrps);
			}
		}
		calcResultDao.save(cr);
		responseVo.setCalcResultId(formulaResultId);
	}
	
	private class RetrieveIndicatorVo {
		protected Timestamp thistime;
		protected List<CalcIndicatorResult> cirs;
		protected long pos;
		protected Set<String> savedFvSet;
	}
	
	private static final int SAVE_SIZE_ONCE = 100;
	
	private void invokeDaoMethod(Object dao, List<?> pos, Method method) {
		try {
			method.invoke(dao, pos);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			throw new FormulaException(e, "执行数据库批量保存出错。");
		}
	}
	
	private void saveBatch(Object dao, Method method, List<?> pos) {
		if(pos.size() <= SAVE_SIZE_ONCE) {
			invokeDaoMethod(dao, pos, method);
			return;
		}
		List<Object> list = new ArrayList<>(SAVE_SIZE_ONCE);
		for(int k = 0;k < pos.size();k ++) {
			if(k > 0 && (k % SAVE_SIZE_ONCE == 0)) {
				invokeDaoMethod(dao, list, method);
				list.clear();
			}
			Object ov = pos.get(k);
			list.add(ov);
		}
		if(list.size() > 0) {
			invokeDaoMethod(dao, list, method);
		}
	}

	private void retrieveIndicators(XbcFmlFormulaVo fv, long parentId, String baseNodeCode, RetrieveIndicatorVo riv) {
		if(riv.savedFvSet.contains(fv.getFormulaCode())) {
			return;
		}
		riv.savedFvSet.add(fv.getFormulaCode());
		List<XbcFmlFormulaIndicatorVo> ivs = fv.getIndicatorVos();
		if(ivs == null || ivs.size() == 0) {
			return;
		}
		ivs = ivs.stream().sorted(Comparator.comparing(XbcFmlFormulaIndicatorVo::getIndicatorName)).collect(Collectors.toList());
		int pos = 0;
		for(XbcFmlFormulaIndicatorVo iv : ivs) {
			String nodeCode = baseNodeCode + ((10000 + pos) + "").substring(1);
			pos ++;
			CalcIndicatorResult cir = new CalcIndicatorResult();
			cir.setKeyId(riv.pos);
			riv.pos = riv.pos + 1;
			cir.setCalcResultId(0);
			cir.setIndicatorId(iv.getKeyId());
			cir.setIndicatorType(iv.getIndicatorType());
			cir.setIndicatorName(iv.getIndicatorName());
			cir.setSpecifiedVal(iv.getSpecifiedVal());
			cir.setCalcResultType(iv.getResult() == null ? "" : iv.getResult().getClass().getName());
			cir.setCalcResult(objToString(iv.getResult()));
			cir.setParentId(parentId);
			cir.setNodeCode(nodeCode);
			cir.setCreateTime(riv.thistime);
			if(iv.getIndicatorType() == XbcFmlFormulaIndicatorVo.FORMULA_TYPE) {
				XbcFmlFormulaVo tmpFv = iv.getFormulaVo();
				cir.setFormulaValue(tmpFv.getFormulaValue());
				retrieveIndicators(tmpFv, cir.getKeyId(), nodeCode, riv);
			} else {
				cir.setFormulaValue("");
			}
			riv.cirs.add(cir);
		}
	}

	private CalcResult createFormulaResult(CalculateRequestVo requestVo, XbcFmlFormulaVo fv, Timestamp thistime) {
		CalcResult cr = new CalcResult();
		cr.setKeyId(0);
		cr.setProductId(requestVo.getProductId());
		cr.setBizType(requestVo.getBizType());
		if(requestVo.getBizCode() != null) {
			cr.setBizCode(requestVo.getBizCode());
		} else {
			cr.setBizCode("");
		}
		cr.setFormulaCode(fv.getFormulaCode());
		cr.setCalcResultType(fv.getResult() == null ? "" : fv.getResult().getClass().getName());
		cr.setCalcResult(objToString(fv.getResult()));
		cr.setFormulaName(fv.getFormulaName());
		cr.setFormulaValue(fv.getFormulaValue());
		cr.setCreateTime(thistime);
		return cr;
	}
	
	private String objToString(Object ov) {
		if(ov == null) {
			return "null";
		}
		Class<?> clazz = ov.getClass();
		if(clazz == BigDecimal.class) {
			BigDecimal bv = (BigDecimal)ov;
			return bv.toPlainString();
		}
		if(clazz == Double.class || clazz == double.class || clazz == Float.class || clazz == float.class) {
			Double dv = (Double)ov;
			BigDecimal bv = new BigDecimal(dv);
			return bv.toPlainString();
		}
		return ov.toString();
	}
}
