package xbc.formula.fmlmgr.logic.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import xbc.formula.engine.exp.XbcExpCalculator;
import xbc.formula.engine.fml.vo.XbcFmlFormulaIndicatorVo;
import xbc.formula.engine.fml.vo.XbcFmlFormulaVo;
import xbc.formula.fmlmgr.dao.FormulaDao;
import xbc.formula.fmlmgr.dao.IndicatorDao;
import xbc.formula.fmlmgr.logic.FormulaLogic;
import xbc.formula.framework.exception.FormulaException;

@Component
public class FormulaLogicImpl implements FormulaLogic {

	@Autowired
	private FormulaDao formulaDao;
	
	@Autowired
	private IndicatorDao indicatorDao;

	@Override
	public XbcFmlFormulaVo getFormulaByCode(String formulaCode) {
		XbcFmlFormulaVo fv = formulaDao.getFormulaByCode(formulaCode);
		if(fv == null) {
			throw new FormulaException("没有找到公式，公式编码: " + formulaCode);
		}
        if(fv.getFormulaValue() == null) {
            throw new FormulaException("公式【" + formulaCode + "】没有配置具体的公式表达式。");
        }
        Set<Long> keyIdSet = new HashSet<>(); // 检查公式循环调用的Set
        retrieveFormula(fv, keyIdSet);
		return fv;
	}

	private void retrieveFormula(XbcFmlFormulaVo fv, Set<Long> keyIdSet) {
        if(keyIdSet.contains(fv.getKeyId())) {
            throw new FormulaException("公式不能循环调用，keyId: " + fv.getKeyId());
        }
        keyIdSet.add(fv.getKeyId());
		List<XbcFmlFormulaIndicatorVo> ivs = new ArrayList<>();
		fv.setIndicatorVos(ivs);
		List<String> indicatorNames = XbcExpCalculator.getParamters(fv.getFormulaValue());
		for(String in : indicatorNames) {
			XbcFmlFormulaIndicatorVo iv = indicatorDao.getIndicatorByName(in);
			if(iv == null) {
				XbcFmlFormulaVo tmpFv = formulaDao.getFormulaByName(in);
				if(tmpFv == null) {
					throw new FormulaException("【" + in + "】不是指标，也不是公式。公式: " + fv.getFormulaValue());
				}
				// 构建一个公式类指标
				iv = new XbcFmlFormulaIndicatorVo();
				iv.setKeyId(tmpFv.getKeyId());
				iv.setIndicatorType(XbcFmlFormulaIndicatorVo.FORMULA_TYPE);
				iv.setIndicatorCode(tmpFv.getFormulaCode());
				iv.setIndicatorName(tmpFv.getFormulaName());
				iv.setSpecifiedVal(tmpFv.getFormulaCode());
				iv.setFormulaVo(tmpFv);
				ivs.add(iv);
				retrieveFormula(tmpFv, keyIdSet);
			} else {
	            ivs.add(iv);
	            if(iv.getIndicatorType() == XbcFmlFormulaIndicatorVo.FORMULA_TYPE) {
	            	XbcFmlFormulaVo tmpFv = formulaDao.getFormulaByCode(iv.getSpecifiedVal());
	            	if(tmpFv == null) {
	            		throw new FormulaException("指标【" + in + "】的[specified_val]配置【" + iv.getSpecifiedVal() + "】不是一个公式。公式: " + fv.getFormulaValue());
	            	}
					iv.setFormulaVo(tmpFv);
					retrieveFormula(tmpFv, keyIdSet);
	            }
			}
		}
        keyIdSet.remove(fv.getKeyId());
	}
}
